import os
import sqlite3
import sys

from . import args
from . import calc
from . import db
from . import parser
from . import util


try:
  try:  # parsing args
    params = args.parse(sys.argv[1:])
  except Exception as e:
    args.print_help()
    raise Exception("failure while parsing args", e)

  try:  # initializing DB
    dbconn = sqlite3.connect(params.dbpath)
    dbcur  = dbconn.cursor()
    db.try_init(dbcur)
    dbcur.execute("BEGIN;")
  except Exception as e:
    raise Exception("failure while initializing DB", e)

  if not sys.stdin.isatty():
    try:  # insert new records
      for tx in parser.parse(sys.stdin):
        db.apply(dbcur, tx)
    except Exception as e:
      raise Exception("failure while inserting new records", e)

  try:  # writing outputs
    calc.call(
      calc.balancesheet,
      params.balancesheet,
      dbcur,
    )
    calc.call(
      calc.plstatement,
      util.get_or(params.plstatement, 0),
      dbcur,
      begin=util.get_or(params.plstatement, 1),
      end  =util.get_or(params.plstatement, 2),
    )
    calc.call(
      calc.balancetransition,
      params.balancetransition,
      dbcur,
    )
    calc.call(
      calc.pltransition,
      util.get_or(params.pltransition, 0),
      dbcur,
      datefmt=util.get_or(params.pltransition, 1),
    )
  except Exception as e:
    raise Exception("failure while calculating output", e)

except Exception as e:
  print(f"bookeeper: error: {e}", file=sys.stderr)
  dbcur and dbcur.execute("ROLLBACK;")
else:
  if params.permanentize:
    dbcur.execute("COMMIT;")
  else:
    dbcur.execute("ROLLBACK;")

finally:
  if dbcur:
    dbcur.close()
  if dbconn:
    dbconn.commit();
    dbconn.close()
